@@ -1,9 +1,22 @@
 ssym.l <-
-function(formula, family, xi, data, epsilon, maxiter, subset, local.influence, spec, std.out){
+function(formula, family, xi, data, epsilon, maxiter, subset, link.mu, link.phi, local.influence, spec, std.out){
 
 if(family!="Normal" & family!="Slash" & family!="Hyperbolic" & family!="Sinh-t" & family!="Sinh-normal" & family!="Contnormal" & family!="Powerexp" & family!="Student")
 stop("family of distributions specified by the user is not supported!!",call.=FALSE)
 
+if(missingArg(link.mu)) link.mu <- "identity"
+if(link.mu=="logarithmic") link.mu <- "log"
+if(link.mu=="exponential") link.mu <- "exp"
+if(link.mu=="reciprocal") link.mu <- "recip"
+if(link.mu!="identity" & link.mu!="log" & link.mu!="exp" & link.mu!="recip") 
+stop("link.mu function specified by the user is not supported!!",call.=FALSE)
+if(missingArg(std.out) || std.out!="FALSE") std.out <- "TRUE"
+
+if(missingArg(link.phi)) link.phi <- "log"
+if(link.phi=="logarithmic") link.phi <- "log"
+if(link.phi!="identity" & link.phi!="log") 
+stop("link.phi function specified by the user is not supported!!",call.=FALSE)
+
 if(family=="Slash" | family=="Hyperbolic" | family=="Sinh-t" | family=="Sinh-normal" | family=="Contnormal" | family=="Powerexp" | family=="Student"){
   if(missingArg(xi)) stop("for the family of distributions specified by the user an extra parameter is required!!", call.=FALSE) 
 }
@@ -77,6 +90,7 @@ if(missingArg(maxiter)) maxiter <- 1000
 	 type.mu <- matrix(1,sum(idx3),1)
 	 type.mu[idx2==1] <- 2
 	 pspm <- matrix(0,n,1)
+	 model.matrix.mu <- matrix(0,n,1)	 
 	 qm <- 0
 	 penm <- matrix(0,1,1)
 	 centm <- matrix(0,1,1)
@@ -94,6 +108,7 @@ if(missingArg(maxiter)) maxiter <- 1000
 		p <- ncol(X)
 		pspm <- cbind(pspm,X)
 		penm <- bdiag(penm,matrix(0,p,p))
+		model.matrix.mu <- cbind(model.matrix.mu,X)
 	 }else{p <- 0}
 	 if(sum(idx3) >= 1){
 		for(i in 1:length(idx3)){
@@ -114,10 +129,14 @@ if(missingArg(maxiter)) maxiter <- 1000
 	    statusm <- statusm[-1]
 		vnpsm <- as.matrix(x[subset,idx3==1])
 		colnames(vnpsm) <- colnames(x)[idx3==1]
+		model.matrix.mu <- cbind(model.matrix.mu,vnpsm)			
 		qm <- qm[-1]
 	 }
 	 pspm <- as.matrix(pspm[,-1])
 	 penm <- as.matrix(penm[-1,-1])
+	 col.m2 <- colnames(model.matrix.mu)[-1]
+	 model.matrix.mu <- as.matrix(model.matrix.mu[,-1])
+	 colnames(model.matrix.mu) <- col.m2
 	 
 	 w <- model.matrix(ll, data)
 	 wa  <- "ncs("	 
@@ -132,6 +151,7 @@ if(missingArg(maxiter)) maxiter <- 1000
 	 type.phi <- matrix(1,sum(idw3),1)
 	 type.phi[idw2==1] <- 2
 	 pspp <- matrix(0,n,1)
+	 model.matrix.phi <- matrix(0,n,1)	 
 	 q <- 0
 	 penp <- matrix(0,1,1)
 	 lambdas.phi <- matrix(0,1,1)		
@@ -149,6 +169,7 @@ if(missingArg(maxiter)) maxiter <- 1000
 		l <- ncol(W)
 		pspp <- cbind(pspp,W)
 		penp <- bdiag(penp,matrix(0,l,l))
+		model.matrix.phi <- cbind(model.matrix.phi,W)
 	 }else{l <- 0}
 	 if(sum(idw3) >= 1){
 		for(i in 1:length(idw3)){
@@ -169,10 +190,14 @@ if(missingArg(maxiter)) maxiter <- 1000
 	statusp <- statusp[-1]
 	vnpsp <- as.matrix(w[subset,idw3==1])
 	colnames(vnpsp) <- colnames(w)[idw3==1]
+    model.matrix.phi <- cbind(model.matrix.phi,vnpsp)		
 	q <- q[-1]
 	}	
 	pspp <- as.matrix(pspp[,-1])
 	penp <- as.matrix(penp[-1,-1])
+	col.m2 <- colnames(model.matrix.phi)[-1]
+	model.matrix.phi <- as.matrix(model.matrix.phi[,-1])
+	colnames(model.matrix.phi) <- col.m2
 	pspp2 <- crossprod(pspp)
 	
 if(family=="Normal"){
@@ -330,10 +355,51 @@ if(family=="Slash"){
 	xix <- 2*integrate(gfg,0,60)$value
 }
 
-objeto <- list(maxiter=maxiter, epsilon=epsilon, y=y, family=family, xi=xi, dg=dg, fg=fg, v=v, p=p, qm=qm, l=l, q=q, pspm=pspm, pspp=pspp, orig="linear")
 
-thetam0 <- solve(crossprod(pspm) + penm)%*%crossprod(pspm,y)
-thetap0 <- solve(crossprod(pspp) + penp)%*%crossprod(pspp,log((y-pspm%*%thetam0)^2/ifelse(is.null(xix),1,xix)))
+if(link.mu=="identity"){
+	l.mu <- function(y) y
+	l.mu.i <- function(y) y
+	l1.mu <- function(mu) matrix(1,length(mu),1)
+	l2.mu <- function(mu) matrix(0,length(mu),1)	
+	attr(l1.mu,"link") <- "identity"}
+else{if(link.mu=="log"){
+		l.mu <- function(y) log(y)
+		l.mu.i <- function(y) exp(y)
+		l1.mu <- function(mu) mu
+		l2.mu <- function(mu) -1/mu^2	
+		attr(l1.mu,"link") <- "logarithmic"}
+	 if(link.mu=="exp"){
+		l.mu <- function(y) exp(y)
+		l.mu.i <- function(y) log(y)
+		l1.mu <- function(mu) 1/mu
+		l2.mu <- function(mu) mu	
+		attr(l1.mu,"link") <- "exponential"}
+	 if(link.mu=="recip"){
+		l.mu <- function(y) 1/y
+		l.mu.i <- function(y) 1/y
+		l1.mu <- function(mu) -mu^2
+		l2.mu <- function(mu) 2/mu^3	
+		attr(l1.mu,"link") <- "reciprocal"}		
+}
+
+if(link.phi=="log"){
+	l.phi <- function(y) log(y)
+	l.phi.i <- function(y) exp(y)
+	l1.phi <- function(phi) matrix(1,length(phi),1)
+	l2.phi <- function(phi) -1/phi^2	
+	attr(l1.phi,"link") <- "logarithmic"}
+else{
+	l.phi <- function(y) y
+	l.phi.i <- function(y) y
+	l1.phi <- function(phi) 1/phi
+	l2.phi <- function(phi) matrix(0,length(phi),1)	
+	attr(l1.phi,"link") <- "identity"}
+	
+
+objeto <- list(maxiter=maxiter, epsilon=epsilon, y=y, l.mu.i=l.mu.i, l.phi.i=l.phi.i, l1.mu=l1.mu, l1.phi=l1.phi, family=family, xi=xi, dg=dg, fg=fg, v=v, p=p, qm=qm, l=l, q=q, pspm=pspm, pspp=pspp, orig="linear")
+
+thetam0 <- solve(crossprod(pspm) + penm)%*%crossprod(pspm,l.mu(y))
+thetap0 <- solve(crossprod(pspp) + penp)%*%crossprod(pspp,l.phi((y-l.mu.i(pspm%*%thetam0))^2/ifelse(is.null(xix),1,xix)))
 theta0 <- c(thetam0,thetap0)
 
 if(any(statusm == "unknown")  | any(statusp == "unknown")){
@@ -361,16 +427,17 @@ if(any(statusm == "unknown")  | any(statusp == "unknown")){
 				     else vP <- itpE(theta0,objeto)
 				}
 
-				mu_es <- pspm%*%vP[1:(p+sum(qm))]
-				phi_es <- exp(pspp%*%vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))])
+				mu_es <- l.mu.i(pspm%*%vP[1:(p+sum(qm))])
+				phi_es <- l.phi.i(pspp%*%vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))])
 				z_es <- (y - mu_es)/sqrt(phi_es)
 
 				gle <- 0
 				if(sum(qm) > 0 ){
-					  pspmw <- pspm*matrix(1/sqrt(phi_es),n,ncol(pspm))
+					  pspmw <- pspm*matrix(l1.mu(mu_es)/sqrt(phi_es),n,ncol(pspm))
 					  gle <- gle + sum(diag(solve(crossprod(pspmw) + (1/dg)*penm2)%*%crossprod(pspmw)))
 				}
 				if(sum(q) > 0 ){
+					  if(attr(l1.phi,"link")!="logarithmic") pspp2 <- crossprod(pspp*matrix(l1.phi(phi_es),n,ncol(pspp)))
 					  gle <- gle + sum(diag(solve(pspp2 + (4/(fg-1))*penp2)%*%pspp2))
 				}
 
@@ -408,20 +475,21 @@ else{if( (family=="Powerexp" && xi < 0) || (family=="Sinh-normal") || (family=="
 
 thetam <- vP[1:(p+sum(qm))]
 thetap <- vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))]
-mu_es <- pspm%*%vP[1:(p+sum(qm))]
-phi_es <- exp(pspp%*%vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))])
+mu_es <- l.mu.i(pspm%*%vP[1:(p+sum(qm))])
+phi_es <- l.phi.i(pspp%*%vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))])
 z_es <- (y - mu_es)/sqrt(phi_es)
 v_es <- v(z_es)
 
-if(missingArg(std.out)){
-	score.mu <- crossprod(pspm,v_es*z_es/sqrt(phi_es)) - penm%*%vP[1:(p+sum(qm))]
-	score.phi <- crossprod(pspp,(v_es*z_es^2 - 1)/2) - penp%*%vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))]
-	pspm2 <- crossprod(pspm,pspm*matrix(1/phi_es,n,ncol(pspm)))
+if(std.out=="TRUE"){
+	score.mu <- crossprod(pspm,l1.mu(mu_es)*v_es*z_es/sqrt(phi_es)) - penm%*%vP[1:(p+sum(qm))]
+	score.phi <- crossprod(pspp,l1.phi(phi_es)*(v_es*z_es^2 - 1)/2) - penp%*%vP[(p+sum(qm)+1):(p+sum(qm)+l+sum(q))]
+	pspm2 <- crossprod(pspm,pspm*matrix(l1.mu(mu_es)^2/phi_es,n,ncol(pspm)))
 	vcov.mu <- solve(dg*pspm2 + penm)
-	if(sum(qm)>0) vcov.mu <- crossprod(vcov.mu,tcrossprod((dg*pspm2),vcov.mu))
+	if(sum(qm)>0) vcov.mu <- crossprod(vcov.mu,tcrossprod(dg*pspm2,vcov.mu))	
+	if(attr(l1.phi,"link")!="logarithmic") pspp2 <- crossprod(pspp,pspp*matrix(l1.phi(phi_es)^2,n,ncol(pspp)))
 	vcov.phi <- solve(((fg-1)/4)*pspp2 + penp)
 	if(sum(q)>0) vcov.phi <- crossprod(vcov.phi,tcrossprod(((fg-1)/4)*pspp2,vcov.phi))
-	
+
 	if(sum(qm) > 0 ){
 	  stes.mu <- matrix(-1,length(qm),2)
 	  pos <- c(0,cumsum(qm)) + p
@@ -459,40 +527,41 @@ if(missingArg(std.out)){
 	}else gle.phi <- l
 
 	out_ <- list(p=p,qm=qm,l=l,q=q,y=y, score.mu=score.mu,score.phi=score.phi,vcov.mu=vcov.mu,vcov.phi=vcov.phi,family=family,xi=xi,
-	               model.matrix.mu=X, model.matrix.phi=W, deviance.mu=deviance.mu(z_es), deviance.phi=deviance.phi(z_es), deviance.mu.f=deviance.mu, deviance.phi.f=deviance.phi, 
+	               model.matrix.mu=model.matrix.mu, model.matrix.phi=model.matrix.phi, deviance.mu=deviance.mu(z_es), deviance.phi=deviance.phi(z_es),
+				   deviance.mu.f=deviance.mu, deviance.phi.f=deviance.phi, 
 				   z_es=z_es, theta.mu=thetam, theta.phi=thetap, cdfz=cdf(z_es), lpdf=(log(pdf(z_es))-(1/2)*log(phi_es)), xix=xix, weights=v_es/dg, censored="FALSE",
-				   dg=dg, fg=fg, mu.fitted=mu_es, phi.fitted=phi_es, v=v, orig="linear", pspm=pspm, pspp=pspp, penm=penm, penp=penp, gle.mu=gle.mu, gle.phi=gle.phi)
+				   dg=dg, fg=fg, mu.fitted=mu_es, phi.fitted=phi_es, v=v, orig="linear", pspm=pspm, pspp=pspp, penm=penm, penp=penp, gle.mu=gle.mu, gle.phi=gle.phi,
+				   l.mu.i=l.mu.i, l.phi.i=l.phi.i, l1.mu=l1.mu, l1.phi=l1.phi)
 	
 	if(sum(qm)>0){
 	  out_$which.mu <- which.mu
 	  out_$lambdas.mu <- lambdas.mu
 	  out_$stes.mu <- stes.mu
 	  out_$type.mu <- type.mu
-	  out_$model.matrix.mu <- cbind(out_$model.matrix.mu,vnpsm)
 	}
 	if(sum(q)>0){
 	  out_$which.phi <- which.phi
 	  out_$lambdas.phi <- lambdas.phi
 	  out_$stes.phi <- stes.phi
 	  out_$type.phi <- type.phi
-	  out_$model.matrix.phi <- cbind(out_$model.matrix.phi,vnpsp)
 	}
 
 if(!missingArg(local.influence)){
 	vp_es <- vp(z_es)
 	Dc   <- (vp_es*z_es + v_es)
 	Dcar <- (vp_es*z_es^2 + 2*z_es*v_es)/2
-	Dcab <- Dcar*z_es/2
+	Dcab <- (Dcar*z_es + (v_es*z_es^2 - 1)*(1 + phi_es^2*l2.phi(phi_es)*l1.phi(phi_es)))/2
+	Dctil <- v_es*z_es*l2.mu(mu_es)*l1.mu(mu_es)
 	Ltt <- matrix(0,ncol(pspm)+ncol(pspp),ncol(pspm)+ncol(pspp))
 	Ltt2 <- matrix(0,ncol(pspm)+ncol(pspp),ncol(pspm)+ncol(pspp))	
-	Ltt[1:ncol(pspm),1:ncol(pspm)] <- -crossprod(pspm,pspm*matrix(Dc/phi_es,n,ncol(pspm))) - penm
-	Ltt[(ncol(pspm)+1):(ncol(pspm)+ncol(pspp)),1:ncol(pspm)] <- -crossprod(pspp,pspm*matrix(Dcar/sqrt(phi_es),n,ncol(pspm)))
+	Ltt[1:ncol(pspm),1:ncol(pspm)] <- -crossprod(pspm,pspm*matrix(l1.mu(mu_es)^2*(Dc/phi_es + Dctil/sqrt(phi_es)),n,ncol(pspm))) - penm
+	Ltt[(ncol(pspm)+1):(ncol(pspm)+ncol(pspp)),1:ncol(pspm)] <- -crossprod(pspp,pspm*matrix(l1.mu(mu_es)*l1.phi(phi_es)*Dcar/sqrt(phi_es),n,ncol(pspm)))
 	Ltt[1:ncol(pspm),(ncol(pspm)+1):(ncol(pspm)+ncol(pspp))] <- t(Ltt[(ncol(pspm)+1):(ncol(pspm)+ncol(pspp)),1:ncol(pspm)])
-	Lgg <- -crossprod(pspp,pspp*matrix(Dcab,n,ncol(pspp))) - penp
+	Lgg <- -crossprod(pspp,pspp*matrix(l1.phi(phi_es)^2*Dcab,n,ncol(pspp))) - penp
 	Ltt[(ncol(pspm)+1):(ncol(pspm)+ncol(pspp)),(ncol(pspm)+1):(ncol(pspm)+ncol(pspp))] <- Lgg
 	Ltt2[(ncol(pspm)+1):(ncol(pspm)+ncol(pspp)),(ncol(pspm)+1):(ncol(pspm)+ncol(pspp))] <- solve(Lgg)
 
-	delta <- rbind(t(pspm*matrix(v_es*z_es/sqrt(phi_es),length(phi_es),ncol(pspm))), (1/2)*t(matrix(v_es*z_es^2-1,length(phi_es),ncol(pspp))*pspp))
+	delta <- rbind(t(pspm*matrix(l1.mu(mu_es)*v_es*z_es/sqrt(phi_es),length(phi_es),ncol(pspm))), (1/2)*t(matrix((v_es*z_es^2-1)*l1.phi(phi_es),length(phi_es),ncol(pspp))*pspp))
 	tl <- t(delta)%*%(solve(Ltt)-Ltt2)%*%delta
 	tl <- tl/sqrt(sum(diag(t(tl)%*%tl)))
 	cw <- abs(eigen(tl,symmetric=TRUE)$vector[,length(phi_es)])
@@ -502,7 +571,7 @@ if(!missingArg(local.influence)){
 	cw.theta <- abs(eigen(tl,symmetric=TRUE)$vector[,length(y)])
 	cw2.theta <- abs(diag(tl))
 	
-	delta <- rbind(t(pspm*matrix(Dc/phi_es,length(phi_es),ncol(pspm))),t(matrix(Dcar/sqrt(phi_es),length(phi_es),ncol(pspp))*pspp))
+	delta <- rbind(t(pspm*matrix(l1.mu(mu_es)*Dc/phi_es,length(phi_es),ncol(pspm))),t(matrix(l1.phi(phi_es)*Dcar/sqrt(phi_es),length(phi_es),ncol(pspp))*pspp))
 	tl2 <- t(delta)%*%(solve(Ltt)-Ltt2)%*%delta
 	tl2 <- tl2/sqrt(sum(diag(t(tl2)%*%tl2)))
 	pr <- abs(eigen(tl2,symmetric=TRUE)$vector[,length(y)])
